#ifndef GUIUTILS_H
#define GUIUTILS_H

#include <gtk/gtk.h>


/*
 *	Button default sizes:
 */
#define GUI_BUTTON_HLABEL_WIDTH			95
#define GUI_BUTTON_HLABEL_HEIGHT		-1

#define GUI_BUTTON_VLABEL_WIDTH			60
#define GUI_BUTTON_VLABEL_HEIGHT		50

/* With GTK_CAN_DEFAULT border */
#define GUI_BUTTON_HLABEL_WIDTH_DEF		(100 + (2 * 3))
#define GUI_BUTTON_HLABEL_HEIGHT_DEF		(30 + (2 * 3))


/*
 *	Menu item default height:
 */
#define GUI_MENU_ITEM_DEF_HEIGHT		20

/*
 *	Menu Item types:
 */
#define GUI_MENU_ITEM_TYPE_LABEL		0	/* Regular menu item */
#define GUI_MENU_ITEM_TYPE_CHECK		1	/* Check menu item */
#define GUI_MENU_ITEM_TYPE_SEPARATOR		2	/* Separator */
#define GUI_MENU_ITEM_TYPE_SUBMENU		3	/* Label that refers
							 * to a submenu
							 */

/*
 *	Menu Bar Item alignment:
 */
#define GUI_MENU_BAR_ALIGN_LEFT			0
#define GUI_MENU_BAR_ALIGN_RIGHT		1
#define GUI_MENU_BAR_ALIGN_CENTER		2


/*
 *	GtkAdjustment Macros:
 */
#define GTK_ADJUSTMENT_GET_VALUE(a)	(((a) != NULL) ? \
 ((GtkAdjustment *)(a))->value : 0.0f)


/*
 *	Unref & Destroy Macros:
 */
#define GDK_COLORMAP_UNREF(p)	\
{ if((p) != NULL) gdk_colormap_unref(p); }
#define GDK_BITMAP_UNREF(p)	\
{ if((p) != NULL) gdk_bitmap_unref(p); }
#define GDK_PIXMAP_UNREF(p)	\
{ if((p) != NULL) gdk_pixmap_unref(p); }
#define GDK_CURSOR_DESTROY(p)	\
{ if((p) != NULL) gdk_cursor_destroy(p); }

#define GTK_RC_STYLE_UNREF(s)		\
{ if((s) != NULL) gtk_rc_style_unref(s); }
#define GTK_STYLE_UNREF(s)		\
{ if((s) != NULL) gtk_style_unref(s); }
#define GTK_ACCEL_GROUP_UNREF(a)	\
{ if((a) != NULL) gtk_accel_group_unref(a); }
#define GTK_WIDGET_DESTROY(w)		\
{ if((w) != NULL) gtk_widget_destroy(w); }
#define GTK_SIGNAL_DISCONNECT(o,i)		\
{ if(((o) != NULL) && ((i) > 0)) {		\
 GtkSignalQuery *q = gtk_signal_query(i);	\
 if(q != NULL) {				\
  g_free(q);					\
  gtk_signal_disconnect(GTK_OBJECT(o), (i));	\
 }						\
} }


/* Block/Unblock Input */
extern void GUIBlockInput(GtkWidget *w, gboolean block);


/* Resize Image Buffer */
extern void GUIResizeBuffer(
	gint bpp,
	const gpointer src,
	gint src_width, gint src_height, gint src_bpl,
	gpointer tar,
	gint tar_width, gint tar_height, gint tar_bpl
);


/* GtkRcStyle Utilities */
#define gtk_rc_style_copy	GUIRCStyleCopy
extern GtkRcStyle *GUIRCStyleCopy(const GtkRcStyle *rcstyle);
#define gtk_widget_modify_style_recursive	GUIRCStyleSetRecursive
extern void GUIRCStyleSetRecursive(GtkWidget *w, GtkRcStyle *rcstyle);


/* Geometry Utilities */
#define gdk_parse_geometry	GUIParseGeometry
extern guint32 GUIParseGeometry(
	const gchar *s,
	gint *x, gint *y, gint *width, gint *height
);
#define gtk_clist_get_cell_geometry	GUICListGetCellGeometry
extern gboolean GUICListGetCellGeometry(
	GtkCList *clist, gint column, gint row,
	gint *x, gint *y, gint *width, gint *height
);
extern gint GUICTreeNodeDeltaRows(
        GtkCTree *ctree,
        GtkCTreeNode *start,    /* Use NULL for first/toplevel node */
        GtkCTreeNode *end
);
#define gdk_window_get_root_position	GUIGetWindowRootPosition
extern void GUIGetWindowRootPosition(
	GdkWindow *window, gint *x, gint *y
);
extern void GUIGetPositionRoot(
	GdkWindow *w, gint x, gint y, gint *rx, gint *ry
);


/* GtkCTree Utilities */
extern gboolean GUICTreeOptimizeExpandPosition(
        GtkCTree *ctree,
        GtkCTreeNode *node      /* Expanded parent node */
);


/* GdkBitmap */
extern GdkBitmap *GUICreateBitmapFromDataRGBA(
	gint width, gint height, gint bpl,
	const guint8 *rgba, guint8 threshold,
	GdkWindow *window
);


/* WM Utilities */
extern void GUISetWMIcon(GdkWindow *window, guint8 **icon_data);


/* Pixmap Utilities */
extern GtkWidget *GUICreateMenuItemIcon(guint8 **icon_data);


/* Widget & Window Mapping */
#define gtk_widget_show_raise   GUIWidgetMapRaise
extern void GUIWidgetMapRaise(GtkWidget *w);
#define gtk_window_minimize     GUIWindowMinimize
extern void GUIWindowMinimize(GtkWindow *window);


/* Tool Tips */
extern void GUISetWidgetTip(GtkWidget *w, const gchar *tip);
extern void GUIShowTipsNow(GtkWidget *w);
extern void GUISetGlobalTipsState(gboolean state);


/* Buttons With Icon & Label */
extern void GUIButtonChangeLayout(
	GtkWidget *w, gboolean show_pixmap, gboolean show_label
);
extern void GUIButtonPixmapUpdate(
	GtkWidget *w, guint8 **icon, const gchar *label
);
extern void GUIButtonArrowUpdate(
	GtkWidget *w,
	GtkArrowType arrow_type, gint arrow_width, gint arrow_height,
	const gchar *label
);
extern GtkWidget *GUIButtonGetMainBox(GtkWidget *w);
extern GtkWidget *GUIButtonGetLabel(GtkWidget *w);
extern GtkWidget *GUIButtonGetPixmap(GtkWidget *w);
extern GtkWidget *GUIButtonGetArrow(GtkWidget *w);
extern gchar *GUIButtonGetLabelText(GtkWidget *w);
extern GtkWidget *GUIButtonPixmap(guint8 **icon);
extern GtkWidget *GUIButtonPixmapLabelH(
	guint8 **icon, const gchar *label, gpointer *label_rtn
);
extern GtkWidget *GUIButtonPixmapLabelV(
	guint8 **icon, const gchar *label, gpointer *label_rtn
);

/* Buttons With Arrow & Label */
extern GtkWidget *GUIButtonArrowLabelH(
	GtkArrowType arrow_type, gint arrow_width, gint arrow_height,
	const gchar *label, gpointer *label_rtn
);
extern GtkWidget *GUIButtonArrowLabelV(
	GtkArrowType arrow_type, gint arrow_width, gint arrow_height,
	const gchar *label, gpointer *label_rtn
);
extern GtkWidget *GUIButtonArrow(
	GtkArrowType arrow_type, gint arrow_width, gint arrow_height
);

/* Toggle Buttons With Icon & Label */
extern GtkWidget *GUIToggleButtonPixmap(guint8 **icon);
extern GtkWidget *GUIToggleButtonPixmapLabelH(
	guint8 **icon, const gchar *label, gpointer *label_rtn
);
extern GtkWidget *GUIToggleButtonPixmapLabelV(
	guint8 **icon, const gchar *label, gpointer *label_rtn
);


/* Prompts With Icon, Label, Entry, and Browse */
extern GtkWidget *GUIPromptBarWithBrowse(
	guint8 **icon, const gchar *label,
	gpointer *label_rtn, gpointer *entry_rtn, gpointer *browse_rtn,
	gpointer browse_data, void (*browse_cb)(GtkWidget *, gpointer)
);
extern GtkWidget *GUIPromptBar(
	guint8 **icon, const gchar *label,
	gpointer *label_rtn, gpointer *entry_rtn
);


/* Dynamic Data Exchange */
extern void GUIDDESetBinary(
	GtkWidget *w,			/* Owner widget */
	GdkAtom selection_atom,		/* Selection */
	gulong t,			/* Time */
	const guint8 *data, gint length
);
extern guint8 *GUIDDEGetBinary(
	GtkWidget *w,			/* Widget making request */
	GdkAtom selection_atom,		/* Can be GDK_NONE */
	gulong t,			/* Time */
	gint *length
);
extern void GUIDDESetString(
	GtkWidget *w,			/* Owner widget */
	GdkAtom selection_atom,		/* Selection */
	gulong t,			/* Time */
	const gchar *data
);
extern gchar *GUIDDEGetString(
	GtkWidget *w,			/* Widget making request */
	GdkAtom selection_atom,		/* Can be GDK_NONE */
	gulong t			/* Time */
);


/* Drag & Drop Utilities */
extern void GUIDNDSetSrc(
	void *w, const void *drag_type, int ndrag_types,
	unsigned int actions, unsigned int buttons,
	void (*begin_cb)(GtkWidget *, GdkDragContext *, gpointer),
	void (*request_data_cb)(GtkWidget *, GdkDragContext *,
		GtkSelectionData *, guint, guint, gpointer),
	void (*delete_data_cb)(GtkWidget *, GdkDragContext *, gpointer),
	void (*end_cb)(GtkWidget *, GdkDragContext *, gpointer),
	gpointer client_data
);
extern void GUIDNDSetTar(
	void *w, const void *drag_type, int ndrag_types,
	unsigned int actions, unsigned int default_action_same,
	unsigned int default_action,
	void (*recieved_data_cb)(GtkWidget *, GdkDragContext *,
		gint, gint, GtkSelectionData *, guint, guint,
		gpointer  
	),
	gpointer client_data
);
extern void GUIDNDSetDragIcon(
	GdkPixmap *pixmap, GdkBitmap *mask, gint hot_x, gint hot_y
);


/* Banner Labels */
extern GtkWidget *GUIBannerCreate(
	const gchar *label, const gchar *font_name,
	GdkColor color_fg, GdkColor color_bg,
	gint align,             /* One of GTK_JUSTIFY_* */
	gboolean expand
);


/* Combo With Label, Combo, Initial Value, and Initial List */
extern GtkWidget *GUIComboCreate(
	const gchar *label,	/* Label */
	const gchar *text,	/* Entry Value */
	GList *list,		/* Combo List */
	gint max_items,		/* Maximum Items In Combo List */
	gpointer *combo_rtn,	/* GtkCombo Return */
	gpointer data,
	void (*func_cb)(GtkWidget *w, gpointer),
	void (*list_change_cb)(GtkWidget *, gpointer, GList *)
);
extern void GUIComboActivateValue(GtkWidget *w, const gchar *value);
extern void GUIComboAddItem(GtkWidget *w, const gchar *value);
extern GList *GUIComboGetList(GtkWidget *w);
extern void GUIComboSetList(GtkWidget *w, GList *list);
extern void GUIComboClearAll(GtkWidget *w);


/* Menu Bar & Menu Items */
extern GtkWidget *GUIMenuBarCreate(GtkAccelGroup **accelgrp_rtn);
extern GtkWidget *GUIMenuCreateTearOff(void);
extern GtkWidget *GUIMenuCreate(void);
extern GtkWidget *GUIMenuItemCreate(
	GtkWidget *menu,
	gint type,		/* One of GUI_MENU_ITEM_TYPE_* */
	GtkAccelGroup *accelgrp,
	guint8 **icon, const gchar *label,
	gint accel_key, guint accel_mods,
	gpointer *functional_widget_rtn,
	gpointer data,
	void (*func_cb)(GtkWidget *w, void *)
);
extern void GUISetMenuItemCrossingCB(
	GtkWidget *w,
	gint (*enter_cb)(GtkWidget *, GdkEvent *, gpointer),
	gpointer enter_data,
	gint (*leave_cb)(GtkWidget *, GdkEvent *, gpointer),
	gpointer leave_data
);
extern GtkWidget *GUIMenuAddToMenuBar(
	GtkWidget *menu_bar, GtkWidget *menu,
	const gchar *label,
	gint align		/* One of GUI_MENU_BAR_ALIGN_* */
);
extern void GUIMenuItemSetLabel(GtkWidget *menu_item, const gchar *label);
extern void GUIMenuItemSetPixmap(GtkWidget *menu_item, guint8 **icon_data);
extern GtkWidget *GUIMenuAddToMenuBarPixmapH(
	GtkWidget *menu_bar, GtkWidget *menu,
	const gchar *label, guint8 **icon_data,
	gint align		/* One of GUI_MENU_BAR_ALIGN_* */
);
extern void GUIMenuItemSetSubMenu(
	GtkWidget *menu_item, GtkWidget *sub_menu
);


/* Pull Out Window */
extern void *GUIPullOutCreateH(
	void *parent_box,
	int homogeneous, int spacing,           /* Of client vbox */
	int expand, int fill, int padding,      /* Of holder hbox */
	int toplevel_width, int toplevel_height,
	void *pull_out_client_data,
	void (*pull_out_cb)(void *, void *),
	void *push_in_client_data,
	void (*push_in_cb)(void *, void *)
);
extern void *GUIPullOutGetToplevelWindow(
	void *client_box,
	int *x, int *y, int *width, int *height
);
extern void GUIPullOutPullOut(void *client_box);
extern void GUIPullOutPushIn(void *client_box);


#endif	/* GUIUTILS_H */
